program Lab3;
{ Use modules}
uses FMM, printer, crt;

procedure GenerateMatrixA( n: Integer; var A: floatmatrix; param: float);
var i, j : Integer;
begin
  for i := 1 to n do
  begin
    for j := 1 to n - 1 do A[ i, j] := powri( j, i - 1);
    A[ i, n] := powri( param, i - 1);
  end;
end;

procedure GenerateVectorB( n: Integer; var B: floatvector);
var i: Integer;
begin
  for i := 1 to n do B[ i] := powri( 2, i) + cos( i);
end;

function GenerateParam( i: Integer): float;
begin
  GenerateParam := 1.0 + 1.0 / powri( 10.0, i);
end;

function GetAposteriorEstimation( n: Integer; X1, X2: floatvector): float;
var
  i     : Integer;
  t1, t2: float;
begin
  t1 := 0; t2 := 0;
  for i := 1 to n do
  begin
    t1 := t1 + powri( X1[ i] - X2[ i], 2);
    t2 := t2 + powri( X1[ i], 2);
  end;
  GetAposteriorEstimation := sqrt( t1 / t2);
end;

procedure MultMatrixMatrix( n: Integer; A, B: floatmatrix; var Res: floatmatrix);
var
  i, j, k: Integer;
begin
  for i := 1 to n do
  begin
    for j := 1 to n do
    begin
      Res[ i, j] := 0.0;
      for k := 1 to n do Res[ i, j] := Res[ i, j] + A[ i, k] * B[ k, j];
    end;
  end;
end;

procedure MultMatrixVector( n: Integer; A: floatmatrix; B: floatvector; var Res: floatvector);
var
  i, j: Integer;
begin
  for i := 1 to n do
  begin
    Res[ i] := 0.0;
    for j := 1 to n do Res[ i] := Res[ i] + A[ i, j] * B[ j];
  end;
end;

procedure PrintMatrix( n: Integer; A: floatmatrix; var pFile: TEXT);
var
  i, j: Integer;
begin
  for i := 1 to n do
  begin
    for j := 1 to n do write( pFile, A[ i, j]:20:7);
    writeln( pFile);
  end;
end;

procedure PrintVector( n: Integer; A: floatvector; var pFile: TEXT);
var
  i: Integer;
begin
  for i := 1 to n do writeln( pFile, A[ i]:20:7);
end;

procedure Transposition( n: Integer; A: floatmatrix; var At: floatmatrix);
var
   i, j: Integer;
begin
  for i:= 1 to n do
    for j := 1 to n do At[ i, j] := A[ j, i];
end;


{ Constants}
const n   = 6; { Dimension of matrix A and B}
      nA  = 7; { Number of matrix A}
      nAt = 5;
Var
  A, A1, At : floatmatrix;
  B1, W, B  : floatvector;
  cond      : float;
  Ip        : ivector;
  param     : float;
  i, j, nRes: Integer;
  pFile     : TEXT;
  ResA      : array [1..nA,  1..n] of float;
  ResAt     : array [1..nAt, 1..n] of float;
Begin
  { Clear screen}
   clrscr;
  { Open file to write result data for A*X=B equation}
  assign ( pFile, 'resultAXB.txt');
  rewrite( pFile);
  for i := 1 to nA do
  begin
   { Get param for matrix A}
   param := GenerateParam( i);
   { Generate initial data}
   GenerateMatrixA( n, A, param);
   GenerateVectorB( n, B);
   { Print initial data "param, A, B"}
   write( pFile, '-----------------------------------------------'); writeln( pFile);
   write( pFile, 'Parameter an = '); writeln( pFile, param);
   writeln( pFile, 'Matrix A:'); PrintMatrix( n, A, pFile);
   writeln( pFile, 'Matrix B:'); PrintVector( n, B, pFile);
   { Calculate solution of equation A*X=B}
   decomp( n, A, cond, Ip, W); { Calculate gaussian matrix}
   solve( n, A, B, Ip);        { Calculate solution, result in B}
   { Print result of solution A*X=B}
   writeln( pFile, 'Result A*X=B:'); PrintVector( n, B, pFile);
   for j:= 1 to n do ResA[ i, j] := B[ j];
   { Aprior estimation of quality}
   write( pFile, 'Aprior estimation (cond) = '); writeln( pFile, cond);
  end;
  close( pFile);
  {Open file to write result data for  At*A*X=At*B equation}
  assign ( pFile, 'resultAtAXAtB.txt');
  rewrite( pFile);
  for i := 1 to nAt do
  begin
   { Get param for matrix A}
   param := GenerateParam( i);
   { Generate initial data}
   GenerateMatrixA( n, A1, param);
   GenerateVectorB( n, B1);
   Transposition( n, A1, At);
   MultMatrixMatrix( n, At, A1, A); { Get At*A}
   MultMatrixVector( n, At, B1, B); { Get At*B}
   { Print initial data "param, A, B"}
   write( pFile, '-----------------------------------------------'); writeln( pFile);
   write( pFile, 'Parameter an = '); writeln( pFile, param);
   writeln( pFile, 'Matrix At*A:'); PrintMatrix( n, A, pFile);
   writeln( pFile, 'Matrix At*B:'); PrintVector( n, B, pFile);
   { Calculate solution of equation At*A*X=At*B}
   decomp( n, A, cond, Ip, W); { Calculate gaussian matrix}
   solve( n, A, B, Ip);        { Calculate solution, result in B}
   { Print result of solution At*A*X=At*B}
   writeln( pFile, 'Result At*A*X=At*B:'); PrintVector( n, B, pFile);
   for j:= 1 to n do ResAt[ i, j] := B[ j];
   { Aprior estimation of quality}
   write( pFile, 'Aprior estimation (cond) = '); writeln( pFile, cond);
  end;
  close( pFile);

  assign ( pFile, 'est.txt');
  rewrite( pFile);
  writeln( pFile, 'Aposterior estimation:');
  if nA > nAt then nRes := nAt
  else nRes := nA;
  for i := 1 to nRes do
  begin
    for j:= 1 to n do B [ j] := ResA[i, j];
    for j:= 1 to n do B1[ j] := ResAt[i, j];
    writeln( pFile, GetAposteriorEstimation( n, B, B1));
  end;
  close( pFile);
end.